﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Media3D;
using System.Collections.Generic;

namespace PerlinNoiseShader.PerlinNoise2D
{
    public class PerlinNoiseShaderEffect:ShaderEffect
    {
        public enum state { single, singleAbs, standard, standardAbs};

        private DependencyProperty g2ArrayProperty;
        private DependencyProperty g2Array2Property;
        private DependencyProperty sizeProperty;
        private DependencyProperty offsetProperty;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        private DependencyProperty amplitudeProperty;
       

        public PerlinNoiseShaderEffect() : this(state.single) { }
        public PerlinNoiseShaderEffect(state eState)
        {
            
            g2ArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("g2Array", typeof(PerlinNoiseShaderEffect), 0);
            g2Array2Property = ShaderEffect.RegisterPixelShaderSamplerProperty("g2Array2", typeof(PerlinNoiseShaderEffect), 1);
            sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(new Size(640, 480), PixelShaderConstantCallback(0)));
            offsetProperty = DependencyProperty.Register("offset", typeof(Point), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(new Point(0, 0), PixelShaderConstantCallback(1)));
            switch(eState){
                case state.single:
                    {
                        c1Property= DependencyProperty.Register("c1", typeof(Color), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(2)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(3)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/PerlinNoiseShader.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(g2ArrayProperty);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.singleAbs:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(2)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(0, 255, 0, 0), PixelShaderConstantCallback(3)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(PerlinNoiseShaderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(4)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/PerlinNoiseShaderAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(g2ArrayProperty);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.standard:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/PerlinNoiseShader2.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(g2ArrayProperty);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }
                case state.standardAbs:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/PerlinNoise2D/PerlinNoiseShader2Abs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(g2ArrayProperty);
                        UpdateShaderValue(sizeProperty);
                        UpdateShaderValue(offsetProperty);
                        break;
                    }    
            }
            
            
            
        }
        
        

        public Brush g2Array
        {
            get { return (Brush)GetValue(g2ArrayProperty); }
            set { SetValue(g2ArrayProperty, value); }
        }

        public Brush g2Array2
        {
            get { return (Brush)GetValue(g2Array2Property); }
            set { SetValue(g2Array2Property, value); }
        }

        public Size size
        {
            get { return (Size)GetValue(sizeProperty); }
            set { SetValue(sizeProperty, value); }
        }

       
        public Point offset
        {
            get { return (Point)GetValue(offsetProperty); }
            set { SetValue(offsetProperty, value); }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }
        
    }
}
