﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using PerlinNoiseShader.ImprovedNoise2D;
using System.Collections.Generic;

namespace PerlinNoiseShader.ImprovedNoise
{
    /// <summary>
    /// consists of methods to calculate single octaves of improved 3D Perlin noise
    /// </summary>
    public class ImprovedNoiseClass:NoiseMask
    {
        
        private ImageBrush oG1ImageBrush;
        private ImageBrush oG2ImageBrush;
        
        private ImageBrush oPrepBrush;
        public WriteableBitmap oPrepBitmap;
        private Rectangle tileRectangle = new Rectangle();
        private PrepTileableNoiseEffect oPrepTileableNoiseEffect = new PrepTileableNoiseEffect();
        private ImprovedNoiseShaderEffect[] aImprovedNoiseShaderEffect;
        private ImprovedNoiseShaderEffect3D[] aImprovedNoiseShaderEffect3D;
        //precalculated images for tileable noise
        private Dictionary<iSize, ImageBrush>  tileDictionary=new Dictionary<iSize,ImageBrush>(); 
                
        public ImprovedNoiseClass():this((new Random()).Next())  {}
        public ImprovedNoiseClass(int random)
        {
            oDrawRectangle = new Rectangle();
            iUIElement = oDrawRectangle;
            aImprovedNoiseShaderEffect = new ImprovedNoiseShaderEffect[4];
            aImprovedNoiseShaderEffect3D = new ImprovedNoiseShaderEffect3D[4];
            for (int i = 0; i < 4; i++)
            {
                aImprovedNoiseShaderEffect[i] = new ImprovedNoiseShaderEffect((ImprovedNoiseShaderEffect.state)i);
                aImprovedNoiseShaderEffect3D[i]=new ImprovedNoiseShaderEffect3D((ImprovedNoiseShaderEffect3D.state)i);
            }
            reInitialize(random);
        }

        

        
        /// <summary>
        /// get the gradient
        /// </summary>
        private void getG()
        {
            int[][] grad3 = {new int[]{-1,-1,0},new int[]{1,-1,0},new int[]{-1,1,0},new int[]{1,1,0},
                    new int[]{-1,0,-1},new int[]{1,0,-1},new int[]{-1,0,1},new int[]{1,0,1},
                    new int[]{0,-1,-1},new int[]{0,1,-1},new int[]{0,-1,1},new int[]{0,1,1},
                    new int[]{-1,-1,0},new int[]{1,-1,0},new int[]{0,-1,1},new int[]{0,1,1}};
            oG1ImageBrush = new ImageBrush();
            oG2ImageBrush = new ImageBrush();
            WriteableBitmap g1Bitmap = new WriteableBitmap(256, 1);
            WriteableBitmap g2Bitmap = new WriteableBitmap(256, 1);
            for (int i = 0; i < 256; i++)
            {
                int perm1 = p[i] % 16;
                int perm2 = p[(i + 1) & 255] % 16;
                int[] vector = grad3[perm1];
                int value1 = (255 << 24) | ((vector[0] + 1) << 16) | ((vector[1] + 1) << 8) | (vector[2] + 1);
                vector = grad3[perm2];
                int value2 = (255 << 24) | ((vector[0] + 1) << 16) | ((vector[1] + 1) << 8) | (vector[2] + 1);
                g1Bitmap.Pixels[i] = value1;
                g2Bitmap.Pixels[i] = value2;
            }
            g1Bitmap.Invalidate();
            g2Bitmap.Invalidate();
            oG1ImageBrush.ImageSource = g1Bitmap;
            oG2ImageBrush.ImageSource = g2Bitmap;
        }



        public void reInitialize()
        {
            reInitialize((new Random()).Next());
        }
        
        /// <summary>
        /// precalculation, if the random seed value should be initialized or changed
        /// </summary>
        /// <param name="random"></param>
        public void reInitialize(int random)
        {
            oRandom = new Random(random);
            thisSeed = random;
            oPBrush = getP();
            getG();
            Rectangle prepRectangle = new Rectangle();
            prepRectangle.Width = 256;
            prepRectangle.Height = 256;
            prepRectangle.Fill = new SolidColorBrush(Color.FromArgb(255, 255, 0, 0));
            PrepImprovedNoiseEffect oPrepImprovedNoiseEffect = new PrepImprovedNoiseEffect();
            prepRectangle.Effect = oPrepImprovedNoiseEffect;
            oPrepImprovedNoiseEffect.pArray = oPBrush;
            oPrepBitmap = new WriteableBitmap(256, 256);
            oPrepBrush = new ImageBrush();
            oPrepBitmap.Render(prepRectangle, null);
            oPrepBitmap.Invalidate();
            oPrepBrush.ImageSource = oPrepBitmap;
            tileRectangle.Width = 256;
            tileRectangle.Height = 256;
            oPrepTileableNoiseEffect.pArray=oPBrush;
            tileRectangle.Fill = new SolidColorBrush(Color.FromArgb(255, 255, 0, 0));
            tileRectangle.Effect = oPrepTileableNoiseEffect;
        }

        /// <summary>
        /// clears the tile dictionary
        /// </summary>
        public void clearTileList(){
            tileDictionary.Clear();
        }

        /// <summary>
        /// calculates a permutation brush for tileable noise
        /// </summary>
        /// <param name="mySize">specifies where to tile at the x and y axis</param>
        /// <returns></returns>
        private ImageBrush getTileBrush(iSize mySize){
            WriteableBitmap oTileBitmap = new WriteableBitmap(256, 256);
            ImageBrush oTileBrush = new ImageBrush();
            oTileBrush.ImageSource = oTileBitmap;
            oPrepTileableNoiseEffect.tileSize = mySize;
            Array.Clear(oTileBitmap.Pixels, 0, oTileBitmap.Pixels.Length);
            oTileBitmap.Render(tileRectangle, null);
            oTileBitmap.Invalidate();
            return oTileBrush;
        }

        /// <summary>
        /// a single octave of flat improved 3D noise
        /// </summary>
        /// <param name="freqXY">frequency at x and y axis</param>
        /// <param name="offset">x and y position of the upper left corner of the xy plane</param>
        /// <param name="z">z position of a xy plane</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleNoise(Size freqXY, Size offset, float z, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }

            ImprovedNoiseShaderEffect oImprovedNoiseShaderEffect=absolute?aImprovedNoiseShaderEffect[1]:aImprovedNoiseShaderEffect[0];
            iUIElement.Effect = oImprovedNoiseShaderEffect;
            oImprovedNoiseShaderEffect.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oImprovedNoiseShaderEffect.c1 = c1;
            oImprovedNoiseShaderEffect.c2 = c2;
            oImprovedNoiseShaderEffect.z = z;
            oImprovedNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oImprovedNoiseShaderEffect.offsetXY = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            oImprovedNoiseShaderEffect.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect.g2Array = oG2ImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
            
        }

        /// <summary>
        /// a single octave of flat improved 3D noise to be mixed with other noise
        /// </summary>
        /// <param name="freqXY">frequency at x and y axis</param>
        /// <param name="offset">x and y position of the upper left corner of the xy plane</param>
        /// <param name="z">z position of a xy plane</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void noise(Size freqXY, Size offset, float z, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }

            ImprovedNoiseShaderEffect oImprovedNoiseShaderEffect = absolute ? aImprovedNoiseShaderEffect[3] : aImprovedNoiseShaderEffect[2];
            iUIElement.Effect = oImprovedNoiseShaderEffect;
            oImprovedNoiseShaderEffect.z = z;
            oImprovedNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oImprovedNoiseShaderEffect.offsetXY = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            oImprovedNoiseShaderEffect.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect.g2Array = oG2ImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        /// <summary>
        /// a single octave of flat improved tileable 3D noise
        /// </summary>
        /// <param name="freqXY">frequency at x and y axis</param>
        /// <param name="tile">if freqXY =(i/width, j/ height), then it tiles at the borders of the image for i=2..256 and j=2..256</param>
        /// <param name="offset">x and y position of the upper left corner of the xy plane</param>
        /// <param name="z">z position of a xy plane</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleNoise(Size freqXY, iSize tile, Size offset, float z, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }
            ImageBrush oTileBrush = null;
            Dictionary<iSize, ImageBrush>.Enumerator oEnumerator = tileDictionary.GetEnumerator();
            while (oEnumerator.MoveNext())
            {
                KeyValuePair<iSize, ImageBrush> myKey = oEnumerator.Current;
                if (myKey.Key.Equals(tile))
                {
                    oTileBrush = myKey.Value;
                    break;
                }
            }
            if (oTileBrush == null)
            {
                oTileBrush = getTileBrush(tile);
                tileDictionary.Add(tile, oTileBrush);
            }
            ImprovedNoiseShaderEffect oImprovedNoiseShaderEffect = absolute ? aImprovedNoiseShaderEffect[1] : aImprovedNoiseShaderEffect[0];
            iUIElement.Effect = oImprovedNoiseShaderEffect;
            oImprovedNoiseShaderEffect.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oImprovedNoiseShaderEffect.c1 = c1;
            oImprovedNoiseShaderEffect.c2 = c2;
            oImprovedNoiseShaderEffect.z = z;
            oImprovedNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oImprovedNoiseShaderEffect.offsetXY = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            oImprovedNoiseShaderEffect.pArray = oTileBrush;
            oImprovedNoiseShaderEffect.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect.g2Array = oG2ImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();

        }

        /// <summary>
        /// a single octave of flat improved tileable 3D noise to be mixed with other noise
        /// </summary>
        /// <param name="freqXY">frequency at x and y axis</param>
        /// <param name="tile">if freqXY =(i/width, j/ height), then it tiles at the borders of the image for i=2..256 and j=2..256</param>
        /// <param name="offset">x and y position of the upper left corner of the xy plane</param>
        /// <param name="z">z position of a xy plane</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void noise(Size freqXY, iSize tile, Size offset, float z, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }
            ImageBrush oTileBrush = null;
            Dictionary<iSize,ImageBrush>.Enumerator oEnumerator=tileDictionary.GetEnumerator();
            while (oEnumerator.MoveNext())
            {
                KeyValuePair<iSize, ImageBrush> myKey = oEnumerator.Current;
                if (myKey.Key.Equals(tile))
                {
                    oTileBrush = myKey.Value;
                    break;
                }
            }
            if (oTileBrush==null)
            {
                oTileBrush = getTileBrush(tile);
                tileDictionary.Add(tile, oTileBrush);
            }
            ImprovedNoiseShaderEffect oImprovedNoiseShaderEffect = absolute ? aImprovedNoiseShaderEffect[3] : aImprovedNoiseShaderEffect[2];
            iUIElement.Effect = oImprovedNoiseShaderEffect;
            oImprovedNoiseShaderEffect.z = z;
            oImprovedNoiseShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oImprovedNoiseShaderEffect.offsetXY = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            oImprovedNoiseShaderEffect.pArray = oTileBrush;
            oImprovedNoiseShaderEffect.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect.g2Array = oG2ImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        /// <summary>
        /// a single octave of custom shaped, improved 3D perlin noise
        /// </summary>
        /// <param name="xPlane">fixed point x values: r.gba</param>
        /// <param name="yPlane">fixed point y values: r.gba</param>
        /// <param name="zPlane">fixed point z values: r.gba</param>
        /// <param name="offset">position of the upper left corner of the shape</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleNoise(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, Vector3d offset, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }

            ImprovedNoiseShaderEffect3D oImprovedNoiseShaderEffect3D = absolute ? aImprovedNoiseShaderEffect3D[1] : aImprovedNoiseShaderEffect3D[0];
            iUIElement.Effect = oImprovedNoiseShaderEffect3D;
            oImprovedNoiseShaderEffect3D.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oImprovedNoiseShaderEffect3D.c1 = c1;
            oImprovedNoiseShaderEffect3D.c2 = c2;
            oImprovedNoiseShaderEffect3D.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect3D.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect3D.g2Array = oG2ImageBrush;
            oImprovedNoiseShaderEffect3D.offsetXY = new Size(offset.x,offset.y);
            oImprovedNoiseShaderEffect3D.offsetZ = (float)offset.z;
            oImprovedNoiseShaderEffect3D.xArray = xPlane;
            oImprovedNoiseShaderEffect3D.yArray = yPlane;
            oImprovedNoiseShaderEffect3D.zArray = zPlane;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        /// <summary>
        /// a single octave of custom shaped, improved 3D perlin noise to be mixed with other noise
        /// </summary>
        /// <param name="xPlane">fixed point x values: r.gba</param>
        /// <param name="yPlane">fixed point y values: r.gba</param>
        /// <param name="zPlane">fixed point z values: r.gba</param>
        /// <param name="mult">multiplication of the xPlane, yPlane and zPlane value</param>
        /// <param name="offset">position of the upper left corner of the shape</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void noise(ImageBrush xPlane, ImageBrush yPlane, ImageBrush zPlane, Vector3d mult, Vector3d offset, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }

            ImprovedNoiseShaderEffect3D oImprovedNoiseShaderEffect3D = absolute ? aImprovedNoiseShaderEffect3D[3] : aImprovedNoiseShaderEffect3D[2];
            iUIElement.Effect = oImprovedNoiseShaderEffect3D;
            oImprovedNoiseShaderEffect3D.pArray = oPrepBrush;
            oImprovedNoiseShaderEffect3D.g1Array = oG1ImageBrush;
            oImprovedNoiseShaderEffect3D.g2Array = oG2ImageBrush;
            oImprovedNoiseShaderEffect3D.offsetXY = new Size(offset.x,offset.y);
            oImprovedNoiseShaderEffect3D.offsetZ = (float)offset.z;
            oImprovedNoiseShaderEffect3D.multXY = new Point(mult.x,mult.y);
            oImprovedNoiseShaderEffect3D.multZ = (float)mult.z;
            oImprovedNoiseShaderEffect3D.xArray = xPlane;
            oImprovedNoiseShaderEffect3D.yArray = yPlane;
            oImprovedNoiseShaderEffect3D.zArray = zPlane;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }
    }
}
