﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.ImprovedNoise
{
    /// <summary>
    /// flat 3D improved perlin noise
    /// </summary>
    public class ImprovedNoiseShaderEffect:ShaderEffect
    {
        public enum state { single, singleAbs, standard, standardAbs };
        private DependencyProperty zProperty;
        private DependencyProperty sizeProperty;
        private DependencyProperty offsetXYProperty;
        private DependencyProperty pArrayProperty;
        private DependencyProperty g1ArrayProperty;
        private DependencyProperty g2ArrayProperty;
        private DependencyProperty amplitudeProperty;
        private DependencyProperty c1Property;
        private DependencyProperty c2Property;
        public ImprovedNoiseShaderEffect() : this(state.single) { }

        public ImprovedNoiseShaderEffect(state eState)
        {
            pArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("pArray", typeof(ImprovedNoiseShaderEffect), 0);
            g1ArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("g1Array", typeof(ImprovedNoiseShaderEffect), 1);
            g2ArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("g2Array", typeof(ImprovedNoiseShaderEffect), 2);
            zProperty = DependencyProperty.Register("z", typeof(float), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(0f, PixelShaderConstantCallback(0)));
            sizeProperty = DependencyProperty.Register("size", typeof(Size), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(new Size(640, 480), PixelShaderConstantCallback(1)));
            offsetXYProperty = DependencyProperty.Register("offsetXY", typeof(Size), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(new Size(0, 0), PixelShaderConstantCallback(2)));
            
            switch (eState)
            {
                case state.single:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(5)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.singleAbs:
                    {
                        c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
                        c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
                        amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(ImprovedNoiseShaderEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(5)));
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoiseAbs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(amplitudeProperty);
                        break;
                    }
                case state.standard:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise2.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        break;
                    }
                case state.standardAbs:
                    {
                        Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/ImprovedPerlinNoise2Abs.ps", UriKind.Relative);
                        this.PixelShader = new PixelShader() { UriSource = u };
                        break;
                    }
                
            }
            UpdateShaderValue(pArrayProperty);
            UpdateShaderValue(g1ArrayProperty);
            UpdateShaderValue(g2ArrayProperty);
            UpdateShaderValue(zProperty);
            UpdateShaderValue(sizeProperty);
            UpdateShaderValue(offsetXYProperty);
        }

        public float z
        {
            get { return (float)GetValue(zProperty); }
            set { SetValue(zProperty, value); }
        }

        public Size size
        {
            get { return (Size)GetValue(sizeProperty); }
            set { SetValue(sizeProperty, value); }
        }

        public Size offsetXY
        {
            get { return (Size)GetValue(offsetXYProperty); }
            set { SetValue(offsetXYProperty, value); }
        }

        public Brush pArray
        {
            get { return (Brush)GetValue(pArrayProperty); }
            set { SetValue(pArrayProperty, value); }
        }

        public Brush g1Array
        {
            get { return (Brush)GetValue(g1ArrayProperty); }
            set { SetValue(g1ArrayProperty, value); }
        }

        public Brush g2Array
        {
            get { return (Brush)GetValue(g2ArrayProperty); }
            set { SetValue(g2ArrayProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

    }
}
