﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using PerlinNoiseShader.ImprovedNoise2D;

namespace PerlinNoiseShader.ImprovedNoise
{
    public class PrepTileableNoiseEffect:ShaderEffect
    {
        public static readonly DependencyProperty pArrayProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("pArray", typeof(PrepTileableNoiseEffect), 0);
        public static readonly DependencyProperty tileProperty = DependencyProperty.Register("tile", typeof(Size), typeof(PrepTileableNoiseEffect), new PropertyMetadata(new Size(256, 256), PixelShaderConstantCallback(0)));
        public static readonly DependencyProperty tile2Property = DependencyProperty.Register("tile2", typeof(Size), typeof(PrepTileableNoiseEffect), new PropertyMetadata(new Size(1.0/256.0, 1.0/256.0), PixelShaderConstantCallback(1)));


        public PrepTileableNoiseEffect()
        {
            Uri u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise/PrepTileableNoise.ps", UriKind.Relative);
            PixelShader = new PixelShader() { UriSource = u };
            
            UpdateShaderValue(pArrayProperty);
            UpdateShaderValue(tileProperty);
            UpdateShaderValue(tile2Property);
            
        }
        
        public Brush pArray
        {
            get { return (Brush)GetValue(pArrayProperty); }
            set { SetValue(pArrayProperty, value); }
        }

        public iSize tileSize
        {
            set
            {
                double x1 = value.Width / 256.0;
                double y1 = value.Height / 256.0;
                double x2 = 256.0 / value.Width;
                double y2 = 256.0 / value.Height;
                SetValue(tileProperty, new Size(x1, y1));
                SetValue(tile2Property, new Size(x2, y2));
            }
            get
            {
                Size oSize = (Size)GetValue(tileProperty);
                oSize.Width *= 256.0;
                oSize.Height *= 256.0;
                return new iSize(oSize);
            }
        }
    }
}
