﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

namespace PerlinNoiseShader.ImprovedNoise2D
{
    public class Noise2DEffect:ShaderEffect
    {
        public enum KindOf { single, singleAbs, singleTileable, singleAbsTileable, standard, standardAbs, standardTileable, standardAbsTileable };
        
        public static readonly DependencyProperty sizeProperty=DependencyProperty.Register("size",typeof(Size),typeof(Noise2DEffect),new PropertyMetadata(new Size(0.05,0.05),PixelShaderConstantCallback(0)));
        public static readonly DependencyProperty offsetProperty = DependencyProperty.Register("offset", typeof(Size), typeof(Noise2DEffect), new PropertyMetadata(new Size(), PixelShaderConstantCallback(1)));
        public static readonly DependencyProperty amplitudeProperty = DependencyProperty.Register("amplitude", typeof(float), typeof(Noise2DEffect), new PropertyMetadata(1f, PixelShaderConstantCallback(2)));
        public static readonly DependencyProperty c1Property = DependencyProperty.Register("c1", typeof(Color), typeof(Noise2DEffect), new PropertyMetadata(Color.FromArgb(255, 0, 0, 0), PixelShaderConstantCallback(3)));
        public static readonly DependencyProperty c2Property = DependencyProperty.Register("c2", typeof(Color), typeof(Noise2DEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(4)));
        public static readonly DependencyProperty tileProperty = DependencyProperty.Register("tile", typeof(Size), typeof(Noise2DEffect), new PropertyMetadata(new Size(256, 256), PixelShaderConstantCallback(5)));
        public static readonly DependencyProperty tile2Property = DependencyProperty.Register("tile2", typeof(Size), typeof(Noise2DEffect), new PropertyMetadata(new Size(256, 256), PixelShaderConstantCallback(6)));
        public static readonly DependencyProperty grad1Property = ShaderEffect.RegisterPixelShaderSamplerProperty("grad1", typeof(Noise2DEffect), 0);
        public static readonly DependencyProperty grad2Property = ShaderEffect.RegisterPixelShaderSamplerProperty("grad2", typeof(Noise2DEffect), 1);
        public static readonly DependencyProperty gradProperty = ShaderEffect.RegisterPixelShaderSamplerProperty("grad", typeof(Noise2DEffect), 2);


        public Noise2DEffect(KindOf eKindOf)
        {
            Uri u;
            switch (eKindOf)
            {
                case KindOf.single:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/noiseShader.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(grad1Property);
                        UpdateShaderValue(grad2Property);
                        break;
                    }
                case KindOf.singleAbs:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/noiseShaderAbs.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(grad1Property);
                        UpdateShaderValue(grad2Property);
                        break;
                    }
                case KindOf.singleTileable:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/pnoiseShader.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(tileProperty);
                        UpdateShaderValue(tile2Property);
                        UpdateShaderValue(gradProperty);
                        break;
                    }
                case KindOf.singleAbsTileable:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/pnoiseShaderAbs.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(amplitudeProperty);
                        UpdateShaderValue(c1Property);
                        UpdateShaderValue(c2Property);
                        UpdateShaderValue(tileProperty);
                        UpdateShaderValue(tile2Property);
                        UpdateShaderValue(gradProperty);
                        break;
                    }
                case KindOf.standard:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/noiseShader2.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(grad1Property);
                        UpdateShaderValue(grad2Property);
                        break;
                    }
                case KindOf.standardAbs:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/noiseShader2Abs.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(grad1Property);
                        UpdateShaderValue(grad2Property);
                        break;
                    }
                case KindOf.standardTileable:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/pnoiseShader2.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(tileProperty);
                        UpdateShaderValue(tile2Property);
                        UpdateShaderValue(gradProperty);
                        break;
                    }
                default:
                    {
                        u = new Uri(@"/PerlinNoiseShader;component/ImprovedNoise2D/pnoiseShader2Abs.ps", UriKind.Relative);
                        PixelShader = new PixelShader() { UriSource = u };
                        UpdateShaderValue(tileProperty);
                        UpdateShaderValue(tile2Property);
                        UpdateShaderValue(gradProperty);
                        break;
                    }
            }
            UpdateShaderValue(sizeProperty);
            UpdateShaderValue(offsetProperty);
            
        }

        public Size size
        {
            get { return (Size)GetValue(sizeProperty); }
            set { SetValue(sizeProperty, value); }
        }

        public Size offset
        {
            get { return (Size)GetValue(offsetProperty); }
            set { SetValue(offsetProperty, value); }
        }

        public float amplitude
        {
            get { return (float)GetValue(amplitudeProperty); }
            set { SetValue(amplitudeProperty, value); }
        }

        public Color c1
        {
            get { return (Color)GetValue(c1Property); }
            set { SetValue(c1Property, value); }
        }

        public Color c2
        {
            get { return (Color)GetValue(c2Property); }
            set { SetValue(c2Property, value); }
        }

        public iSize tile
        {
            get
            {
                Size oSize = ((Size)GetValue(tileProperty));
                oSize.Width = 1.0 / oSize.Width;
                oSize.Height = 1.0 / oSize.Height;
                return new iSize(oSize);
            }
            set
            {
                Size oSize = value.getSize();
                Size oSize2 = new Size(1.0 / oSize.Width, 1.0 / oSize.Height);
                oSize.Width /= 256.0;
                oSize.Height /= 256.0;
                SetValue(tileProperty, oSize);
                SetValue(tile2Property, oSize2);
            }
        }

        public Brush grad1
        {
            get { return (Brush)GetValue(grad1Property); }
            set { SetValue(grad1Property, value); }
        }

        public Brush grad2
        {
            get { return (Brush)GetValue(grad2Property); }
            set { SetValue(grad2Property, value); }
        }

        public Brush grad
        {
            get { return (Brush)GetValue(gradProperty); }
            set { SetValue(gradProperty, value); }
        }
        
    }

    public class iSize
    {
        private int width = 2, height = 2;
        public iSize() { }
        public iSize(int width, int height)
        {
            this.width = width > 2 ? width < 256 ? width : 256 : 2;
            this.height = height > 2 ? height < 256 ? height : 256 : 2;
        }
        
        public iSize(Size oSize){
            if(oSize!=null){
                width=(int)oSize.Width;
                height=(int)oSize.Height;
                width = width > 2 ? width < 256 ? width : 256 : 2;
                height = height > 2 ? height < 256 ? height : 256 : 2;
            }
        }

        public int Width
        {
            set { width = value > 2 ? value < 256 ? value : 256 : 2; }
            get { return width; }
        }

        public int Height
        {
            set { height = value > 2 ? value < 256 ? value : 256 : 2; }
            get { return height; }
        }
        
        public Size getSize()
        {
            return new Size(width, height);
        }

        public override bool Equals(System.Object obj)
        {
            // If parameter is null return false.
            if (obj == null)
            {
                return false;
            }

            // If parameter cannot be cast to Point return false.
            iSize size = obj as iSize;
            if ((System.Object)size == null)
            {
                return false;
            }

            // Return true if the fields match:
            return (width == size.width) && (height == size.height);
        }


    }
}
