﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using PerlinNoiseShader;

namespace PerlinProject.SphereMapping
{
    /// <summary>
    /// calculates a lighted sphere
    /// </summary>
    public class SphereLightEffect:ShaderEffect
    {
        public readonly DependencyProperty difLightProperty = DependencyProperty.Register("difLight", typeof(Color), typeof(SphereLightEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(0)));
        public readonly DependencyProperty specLightProperty = DependencyProperty.Register("specLight", typeof(Color), typeof(SphereLightEffect), new PropertyMetadata(Color.FromArgb(255, 255, 255, 255), PixelShaderConstantCallback(1)));
        public readonly DependencyProperty lightDirectionXYProperty = DependencyProperty.Register("lightDirectionXY", typeof(Point), typeof(SphereLightEffect), new PropertyMetadata(new Point(0, 0), PixelShaderConstantCallback(2)));
        public readonly DependencyProperty lightDirectionZProperty = DependencyProperty.Register("lightDirectionZ", typeof(float), typeof(SphereLightEffect), new PropertyMetadata(0f, PixelShaderConstantCallback(3)));
        public readonly DependencyProperty specularProperty = DependencyProperty.Register("specular", typeof(float), typeof(SphereLightEffect), new PropertyMetadata(0f, PixelShaderConstantCallback(4)));
        public SphereLightEffect()
        {
            Uri u = new Uri(@"/PerlinProject;component/SphereMapping/SphereLight.ps", UriKind.Relative);
            PixelShader = new PixelShader() { UriSource = u };
            UpdateShaderValue(difLightProperty);
            UpdateShaderValue(specLightProperty);
            UpdateShaderValue(lightDirectionXYProperty);
            UpdateShaderValue(lightDirectionZProperty);
            UpdateShaderValue(specularProperty);
        }

        public Color difLight
        {
            get { return (Color)GetValue(difLightProperty); }
            set { SetValue(difLightProperty, value); }
        }

        public Color specLight
        {
            get { return (Color)GetValue(specLightProperty); }
            set { SetValue(specLightProperty, value); }
        }

        public Vector3d lightDirection
        {
            get { Point oPoint = (Point)GetValue(lightDirectionXYProperty);
                   float z=(float)GetValue(lightDirectionZProperty);
                return new Vector3d(oPoint.X,oPoint.Y,z);
            }
            set {Vector3d oVector3d=value;
                SetValue(lightDirectionXYProperty, new Point(oVector3d.x,oVector3d.y));
                SetValue(lightDirectionZProperty, (float)oVector3d.z);
            }
        }
        public float specular
        {
            get { return (float)GetValue(specularProperty); }
            set { SetValue(specularProperty, value); }
        }
    }
}
